/*
    Copyright 2008 Piotr Domagala and Jakub Mielczarek
    This file is part of the Filtertool.

    Filtertool is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * @author Piotr Domagala
  * @author Jakub Mielczarek
  */

#include <boost/program_options.hpp>
#include <iostream>
#include "DataOperations.h"
#include "DataHandler.h"


namespace po = boost::program_options;
int main(int argc, char* argv[])
{
  bool created = false;
  DataHandler *a;
  std::string command="";
  for(int i=0;i<argc;i++)  
    command=command+argv[i]+" ";
  try
  {	
    po::options_description argless( "Options without arguments");
    argless.add_options()
      ( "help,h", "standard help" )
      ( "version,v", "display version" )
      ( "float", "Use float as data type" )
      ( "int", "Use int as data type" )
      ( "join,j", "join input" );
      po::options_description withargs( "Options with arguments" );
    std::string files_in;
    std::string file_out;
    withargs.add_options()
      ( "input-files,i", po::value< std::string>(&files_in), "give the names of files separated by \',\'" )
      ( "output-file,o", po::value< std::string >(&file_out), "specifies otupt files names separated by \',\'. Without this option all will be printed to standard output")
      ( "swap,s", po::value< std::string >(), "swaps two columns in selected file: - s 1,2,3 means swap first and second column in third file")
      ( "mean,m", po::value< char >(), "calculate mean in columns (-m c) or lines (-m l) ")
      ( "filtr,f", po::value< std::string >() , "use this option to select values, type 'man prog' for more details");
    po::options_description helpInfo( "Available options" );
    helpInfo.add( argless ).add( withargs );

    po::options_description parseCmdLine;
    parseCmdLine.add( argless ).add( withargs );
    po::variables_map vm;
    try
    {
      store( po::command_line_parser(argc,argv).options(parseCmdLine).run(), vm );
    }
    catch( std::exception& e_ )
      {
	std::cerr << "Error while parsing command-line: " << e_.what() << "\n";
	exit(-1);
      }
    po::notify( vm );
    if( vm.count("int") )
    {
      std::clog<<"Using Integers as data type"<<std::endl;
      a = new DataOperations<int>();
      created = true;
    }
    else if(vm.count("float") )
    {
      std::clog<<"Using Float as data type"<<std::endl;
      a = new DataOperations<float>();
      created = true;
    }
    if (!created)
    {
      a = new DataOperations<double>();
    }
    if( vm.count("help") || argc == 1 )
    {
      std::cout << "usage: ./program [OPTION <arg1,arg2,...>] ...\n\n";
      std::cout << helpInfo << "\n";
      return 0;
    }
    if( vm.count("version") )
    {
      std::cout << "version 1.0\n";
      return 0;
    }
    if( vm.count("number") )
    {
      std::cout << "version 1.0\n";
      return 0;
    }
    if( vm.count("input-files") )
    {
      a->parseFiles(files_in);
    }
    else if(vm.count("input-files") == 0)
    {
      std::cout<<"read from stdin"<<std::endl;
      a->load();
    }
    if( vm.count("mean") )
    {
      a->mean(vm["mean"].as<char>());
    }
    if( vm.count("output-file") )
    {
      a->setOutputFiles(vm["output-file"].as<std::string>());
    }
    if( vm.count("swap") )
    {
     a->swap(vm["swap"].as<std::string>());
    }
    if( vm.count("join") )
    {
      a->join();
     }
    if( vm.count("filtr") )
    {
      a->setFilter(vm["filtr"].as< std::string >());	
    }
  }
  catch( std::exception& e_ )
    {
      std::cerr << "While defining program options: " << e_.what() << "\n";
      return 1;
    }
  a->setCommand(command);
  a->print();
  delete a;
  return 0;
}
